/*		 ICP-PCI Kernel module free public release
 *
 *		       Copyright (C) 1996-99 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: amdigReg.h,v 5.1.2.1 1999/09/22 11:46:34 matthias Exp $
 *
 */

#ifndef _AMDIGREG_H
#define _AMDIGREG_H

#if defined(linux) && defined(__KERNEL__) || defined(__CONFIG__)

/* AM-DIG Hardware Reference Manual; Imaging Technology Inc. */
/* Document Number 47-H44002-00; October 1993 */

/* R-O   Read-Only Register */
/* W-O   Write-Only Register */
/* R/W   Read/Write Register */
/* R/W1C Clear on Write One to Register */
/* R/W0C Clear on Write Zero to Register */

/* AM-DIG Register Set (16bit access only), pages 3-1 to 3-21 */
/* ********************************************************** */

/* 
 * Prototypes
 */

/* read handler */

u_char dg_get_rev (u_char);   
u_char dg_get_modver (u_char);
u_char dg_get_fsel (u_char);  
u_char dg_get_clkdiv (u_char);
u_char dg_get_xillen (u_char);
u_char dg_get_mc (u_char);    
u_char dg_get_pclkpol (u_char);
u_char dg_get_lenpol (u_char);
u_char dg_get_fenpol (u_char);
u_char dg_get_fldpol (u_char);
u_char dg_get_smode (u_char); 
u_char dg_get_ilmode (u_char);
u_char dg_get_lensm (u_char); 
u_char dg_get_nolmiss (u_char);
u_char dg_get_lvar (u_char);  
u_char dg_get_exclk (u_char); 
u_char dg_get_extmd (u_char); 
u_char dg_get_exsen (u_char); 
u_char dg_get_exspol (u_char); 
u_char dg_get_prien  (u_char);
u_char dg_get_prip (u_char);  
u_short dg_get_extime (u_char);
u_char dg_get_psize (u_char); 
u_char dg_get_ilpixel (u_char);
u_char dg_get_muxa (u_char);  
u_char dg_get_muxb (u_char);  
u_char dg_get_muxc (u_char);  
u_char dg_get_lutpg (u_char); 
u_char dg_get_olutpg (u_char);
u_short dg_get_hoff (u_char); 
u_short dg_get_hact (u_char); 
u_short dg_get_voff (u_char); 
u_short dg_get_vact (u_char); 
u_char dg_get_trigen (u_char);
u_char dg_get_trigsrc (u_char);
u_char dg_get_trigcy (u_char);
u_char dg_get_vb (u_char);    
u_char dg_get_fld (u_char);   
u_short dg_get_lstat (u_char);
u_char dg_get_lut16 (u_char, union dg_lut16_t*);
u_char dg_get_lut8 (u_char, struct dg_lut8_t*);

/* write handler */

int dg_set_fsel (u_char, u_char);  
int dg_set_clkdiv (u_char, u_char);
int dg_set_xillen (u_char, u_char);
int dg_set_mc (u_char, u_char);    
int dg_set_pclkpol (u_char, u_char);
int dg_set_lenpol (u_char, u_char);
int dg_set_fenpol (u_char, u_char);
int dg_set_fldpol (u_char, u_char);
int dg_set_smode (u_char, u_char); 
int dg_set_ilmode (u_char, u_char);
int dg_set_lensm (u_char, u_char); 
int dg_set_nolmiss (u_char, u_char);
int dg_set_lvar (u_char, u_char);  
int dg_set_exclk (u_char, u_char); 
int dg_set_extmd (u_char, u_char); 
int dg_set_exsen (u_char, u_char); 
int dg_set_exspol (u_char, u_char); 
int dg_set_prien  (u_char, u_char);
int dg_set_prip (u_char, u_char);  
int dg_set_extime (u_char, u_short);
int dg_set_psize (u_char, u_char); 
int dg_set_ilpixel (u_char, u_char);
int dg_set_muxa (u_char, u_char);  
int dg_set_muxb (u_char, u_char);  
int dg_set_muxc (u_char, u_char);  
int dg_set_lutpg (u_char, u_char); 
int dg_set_olutpg (u_char, u_char);
int dg_set_hoff (u_char, u_short); 
int dg_set_hact (u_char, u_short); 
int dg_set_voff (u_char, u_short); 
int dg_set_vact (u_char, u_short); 
int dg_set_trigen (u_char, u_char);
int dg_set_trigsrc (u_char, u_char);
int dg_set_lut16 (u_char, union dg_lut16_t*);
int dg_set_lut8 (u_char, struct dg_lut8_t*);

/* init handler */

int dg_init_clkcon (u_char);
int dg_init_ctcon (u_char);
int dg_init_excon (u_char);
int dg_init_lutcon (u_char);
int dg_init_lutsel (u_char);
int dg_init_trgcon (u_char);

/* misc handler */

int dg_sw_trig (u_char);
int dg_lin_lut8 (u_char);
int dg_lin_lut16 (u_char, u_char);
void dg_dump_regs (u_char);


#define DG_MODE_LINEAR		0
#define DG_MODE_NORM_TO8	1
#define DG_MODE_NORM_TO16	2

/* end of prototypes */

#define DG_DISABLE		0
#define DG_ENABLE		1

#define DG_REG_SIZE		15	/* size of register set */

#define DG_NO_MASK		0xFFFF	/* no checking */
#define DG_NO_MIN		0X0000
#define DG_NO_MAX		0xFFFF

#define DG_RES_INIT		0x0000	/* reset reserved bits to zero */

/*    u_short	  modid		Module Identification */
/* -------------------------------------------------- */

#define DG_MODID_OFFSET		0x00

/*    u_short      rev:3         Revision R-O */

#define DG_REV_MASK		0x0007
#define DG_REV_SHIFT		0

/*    u_short      type:5        Board Type R-O */

/*    unified access interface for every module type, see icpciAux.c */

/*    u_short      modver	Module Version R-O */
/* ----------------------------------------------- */

#define DG_MODVER_OFFSET	0x01
#define DG_MODVER_MIN		DG_NO_MIN
#define DG_MODVER_MAX		0x0F
#define DG_MODVER_MASK		DG_MODVER_MAX

#define DG_MODVER_16_RS422	0	/* 16 bit / RS-422 Interface */
#define DG_MODVER_16_TTL	1	/* 16 bit /    TTL Interface */
#define DG_MODVER_24_TTL	2	/* 24 bit /    TTL Interface */
#define DG_MODVER_8_RS422	3	/*  8 bit / RS-422 Interface */
#define DG_MODVER_8_TTL		4	/*  8 bit /    TTL Interface */

/*    u_short      clkcon	Camera Clock Control */
/* ------------------------------------------------- */

#define DG_CLKCON_OFFSET	0x02
#define DG_CLKCON_INIT		0x0000
				/* 14.32 MHz */
				/* Clock Divisor 1 */
				/* Xillix Mode Disable */
				/* Mode Control 0 */

/*    u_short      fsel:4	Camera Clock Frequency Select R/W */

#define DG_FSEL_MASK		0x000F
#define DG_FSEL_SHIFT		0
#define DG_F_14_32		0	/* 14.32 MHz */
#define DG_F_16_26		1	/* 16.26 MHz */
#define DG_F_32_51		3	/* 32.51 MHz */
#define DG_F_25_18_I		4	/* 25.18 MHz */
#define DG_F_28_32_I		5	/* 28.32 MHz */
#define DG_F_24_00		6	/* 24.00 MHz */
#define DG_F_40_00		7	/* 40.00 MHz */
#define DG_F_25_18_II		8	/* 25.18 MHz (same as #4) */
#define DG_F_28_32_II		9	/* 28.32 MHz (same as #5) */
#define DG_F_36_00		10	/* 36.00 MHz */
#define DG_F_44_90		12	/* 44.90 MHz */
#define DG_F_50_00		13	/* 50.00 MHz */

/*    u_short      res_1:1	reserved - reset to 0 */

/*    u_short      clkdiv:3	Camera Clock Divisor R/W */

#define DG_CLKDIV_MASK		0x00E0
#define DG_CLKDIV_SHIFT		5
#define DG_D_1			0	/* clock divided by 1 */
#define DG_D_2			1	/* clock divided by 2 */
#define DG_D_4			2	/* clock divided by 4 */
#define DG_D_8			3	/* clock divided by 8 */
#define DG_D_16			4	/* clock divided by 16 */
#define DG_D_32			5	/* clock divided by 32 */
#define DG_D_64			6	/* clock divided by 64 */
#define DG_D_128		7	/* clock divided by 128 */

/*    u_short      xilen:1	Xillix Mode Enable R/W */

#define DG_XILLEN_MASK		0x0100
#define DG_XILLEN_SHIFT		8

/*    u_short      mc:1		Mode Control R/W */

#define DG_MC_MASK		0x0E00
#define DG_MC_SHIFT		9
#define DG_M_0			0	/* Camera Operation Mode 0 */
#define DG_M_1			1	/* Camera Operation Mode 1 */
#define DG_M_2			2	/* Camera Operation Mode 2 */
#define DG_M_3			3	/* Camera Operation Mode 3 */
#define DG_M_4			4	/* Camera Operation Mode 4 */
#define DG_M_5			5	/* Camera Operation Mode 5 */
#define DG_M_6			6	/* Camera Operation Mode 6 */
#define DG_M_7			7	/* Camera Operation Mode 7 */

/*    u_short      res_1:4	reserved - reset to 0 */

/*    u_short      ctcon		Camera Timing Control */
/* -------------------------------------------------- */

#define DG_CTCON_OFFSET		0x03
#define DG_CTCON_INIT		0x0104
				/* PCLK: sample on rising edge */
				/* LEN: load on rising edge */
				/* FEN: load on rising edge */
				/* FLD: low defines even field */
				/* Area Scan Mode */
				/* Non - interlaced Camera */
				/* LEN inactive grater than PCLK */
				/* Two Lines inserted between line scan (ICP) */
				/* normal FIFO operation in VB */

/*    u_short      pclkpol:1	Pixel Clock Poarity R/W */

#define DG_PCLKPOL_MASK		0x0001
#define DG_PCLKPOL_SHIFT	0
#define DG_PCLK_RI		0	/* sample on rising edge  */
#define DG_PCLK_FA		1	/* sample on falling edge */

/*    u_short      lenpol:1	Line Enable Polarity R/W */

#define DG_LENPOL_MASK		0x0002
#define DG_LENPOL_SHIFT		1
#define DG_LEN_RI		0	/* load on rising edge */
#define DG_LEN_FA		1	/* load on falling edge */

/*    u_short      fenpol:1	Frame Enable Polarity R/W */

#define DG_FENPOL_MASK		0x0004
#define DG_FENPOL_SHIFT		2
#define DG_FEN_FA		0	/* load on falling edge */
#define DG_FEN_RI		1	/* load on rising edge */

/*    u_short      fldpol:1	Field Polarity R/W */

#define DG_FLDPOL_MASK		0x0008
#define DG_FLDPOL_SHIFT		3
#define DG_FLD_LO		0	/* low defines even field */
#define DG_FLD_HI		1	/* high defines even filed */

/*    u_short      smode:1	Scan Mode R/W */

#define DG_SMODE_MASK		0x0010
#define DG_SMODE_SHIFT		4
#define DG_AREA_SCAN		0	/* area scan mode */
#define DG_LINE_SCAN		1	/* line scan mode */

/*    u_short      ilmode:1	Interlace Mode R/W */

#define DG_ILMODE_MASK		0x0020
#define DG_ILMODE_SHIFT		5	
#define DG_NON_ILACE		0	/* non - interlaced camera */
#define DG_ILACE		1	/* interlaced camera */

/*    u_short      lensm:1	Line Enable Small R/W */

#define DG_LENSM_MASK		0x0040
#define DG_LENSM_SHIFT		6	
#define DG_LEN_WIDE		0	/* LEN inactive is grater than pclk */
#define DG_LEN_SMALL		1	/* LEN inactive is smaller than pclk */

/*    u_short      res_1:1	reserved - reset to 0 */

/*    u_short      nolmiss:1	No Line Miss R/W */

#define DG_NOLMISS_MASK		0x0100
#define DG_NOLMISS_SHIFT	8	
#define DG_COMPAT_MFG		0	/* 0 lines inserted between line scan */
#define DG_COMPAT_ICP		1	/* 2 lines inserted between line scan */

/*    u_short      lvar:1	Line Variable R/W */

#define DG_LVAR_MASK		0x0200
#define DG_LVAR_SHIFT		9	
#define DG_FIFO_NORMAL		0	/* normal FIFO operation during VB */
#define DG_FIFO_RESET		1	/* reset FIFO during VB */

/*    u_short      res_2:6	reserved - reset to 0 */

/*    u_short      excon		External Sync Control */
/* -------------------------------------------------- */

#define DG_EXCON_OFFSET		0x04
#define DG_EXCON_INIT		0x003D
				/* 7KHz */ 
				/* Vertical Blank Trigger */ 
				/* Counter Enabled */ 
				/* goes low when triggered */ 
				/* Programmable Integration Disabled */ 
				/* PRI asserted low */ 

/*    u_short      exclk:3	External Sync Clock Frequency R/W */

#define DG_EXCLK_MASK		0x0007
#define DG_EXCLK_SHIFT		0
#define DG_C_7_16_M		0	/*   7.16  MHz / 140    ns */
#define DG_C_1_79_M		1	/*   1.79  MHz / 558.6  ns */
#define DG_C_447_5_k		2	/* 447.5   kHz /   2.23 us */
#define DG_C_112_k		3	/* 112     kHz /   8.93 us */
#define DG_C_28_k		4	/*  28     kHz /  35.7  us */
#define DG_C_7_k		5	/*   7     kHz / 143    us */

/*    u_short      extmd:2	External Sync Mode R/W */

#define DG_EXTMD_MASK		0x0018
#define DG_EXTMD_SHIFT		3
#define DG_FREE_RUN		0	/* Free Running */
#define DG_EXT_TRIG		1	/* External Trigger */
#define DG_SOFT_TRIG		2	/* Software Trigger */
#define DG_VB_TRIG		3	/* Vertical Blank Trigger */

/*    u_short      exsen:1	External Sync Enable R/W */

#define DG_EXSEN_MASK		0x0020
#define DG_EXSEN_SHIFT		5

/*    u_short      exspol:1	External Sync Polarity R/W */

#define DG_EXSPOL_MASK		0x0040
#define DG_EXSPOL_SHIFT		6
#define DG_EXS_LO		0	/* goes low when triggered */
#define DG_EXS_HI		1	/* goes high when triggered */

/*    u_short      prien:1	Programmable Integration Enable R/W */

#define DG_PRIEN_MASK		0x0080
#define DG_PRIEN_SHIFT		7

/*    u_short      prip:1        Programmable Inegration Pulse Polarity R/W */

#define DG_PRIP_MASK		0x0100
#define DG_PRIP_SHIFT		8
#define DG_PRI_LO		0	/* asserted low */
#define DG_PRI_HI		1	/* asserted high */

/*    u_short      res_1:7	reserved - reset to 0 */

/*    u_short      extime	External Sync Time R/W */

#define DG_EXTIME_OFFSET	0x05
#define DG_EXTIME_MASK		0x0FFF
#define DG_EXTIME_MIN		DG_NO_MIN
#define DG_EXTIME_MAX		DG_EXTIME_MASK
#define DG_EXTIME_INIT		0x800	/* 2048 in decimal */

/*    u_short      lutcon	Lookup Table Control */
/* ------------------------------------------------- */

#define DG_LUTCON_OFFSET	0x06
#define DG_LUTCON_INIT		0x0000
				/* 8 bit Input */ 
				/* Non- Interleaved Pixels */ 
				/* Low Byte of LUT to AMOutA */ 
				/* Low Byte of LUT to AMOutB */ 
				/* Low Byte of LUT to AMOutC */ 

/*    u_short      res_1:1	reserved - reset to 0 */

/*    u_short      psize:2	Input Pixel Size R/W */

#define DG_PSIZE_MASK		0x0006
#define DG_PSIZE_SHIFT		1
#define DG_PSIZE_8		0	/*  8-bit */
#define DG_PSIZE_10		1	/* 10-bit */
#define DG_PSIZE_12		2	/* 12-bit */
#define DG_PSIZE_16		3	/* 16-bit */
#define DG_PSIZE_8_MASK		0xFF	/* 256 pages available */
#define DG_PSIZE_8_SHIFT	0	/* page size 256 (1 * 256) */
#define DG_PSIZE_10_MASK	0xFC	/* 64 pages available */
#define DG_PSIZE_10_SHIFT	2	/* page size 1024 (4 * 256) */
#define DG_PSIZE_12_MASK	0xF0	/* 16 pages available */
#define DG_PSIZE_12_SHIFT	4	/* page size 4096 (16 * 256) */
#define DG_PSIZE_16_MASK	0x00	/* 1 page available */
#define DG_PSIZE_16_SHIFT	8	/* page size 65536 (256 * 256) */

/*    u_short      res_2:1	reserved - reset to 0 */

/*    u_short      ilpixel:1	Interleaved Pixl Mode R/W */

#define DG_ILPIXEL_MASK		0x0010
#define DG_ILPIXEL_SHIFT	4
#define DG_NON_ILEAVE		0	/* Non - Interleaved Pixels */
#define DG_ILEAVE		1	/* Interleaved Pixels */

/*    u_short      muxa:1	A - Multiplexer R/W */

#define DG_MUXA_MASK		0x0020
#define DG_MUXA_SHIFT		5
#define DG_MUXA_LO		0	/* Low Byte of LUT to AMOutA */
#define DG_MUXA_HI		1	/* High Byte of LUT to AMOutA */

/*    u_short      muxb:1	B - Multiplexer R/W */

#define DG_MUXB_MASK		0x0040
#define DG_MUXB_SHIFT		6
#define DG_MUXB_LO		0	/* Low Byte of LUT to AMOutB */
#define DG_MUXB_HI		1	/* High Byte of LUT to AMOutB */

/*    u_short      muxc:1	C - Multiplexer R/W */

#define DG_MUXC_MASK		0x0080
#define DG_MUXC_SHIFT		7
#define DG_MUXC_LO		0	/* Low Byte of LUT to AMOutC */
#define DG_MUXC_HI		1	/* High Byte of LUT to AMOutC */

/*    u_short      res_3:8	reserved - reset to 0 */

/*    u_short      lutsel	Lookup Table Page Select */
/* ----------------------------------------------------- */

#define DG_LUTSEL_OFFSET	0x07
#define DG_LUTSEL_INIT		0x0000
				/*  */ 
				/*  */ 

/*    u_short      lutpg:8	Input LUT Page Select R/W */

#define DG_LUTPG_MASK		0x00FF
#define DG_LUTPG_SHIFT		0
#define DG_LUTPG_MIN		DG_NO_MIN
#define DG_LUTPG_MAX		DG_LUTPG_MASK >> DG_OLUTPG_SHIFT

/*    u_short      olutpg:3	Optional LUT Page Select R/W */

#define DG_OLUTPG_MASK		0x0700
#define DG_OLUTPG_SHIFT		8
#define DG_OLUTPG_MIN		DG_NO_MIN
#define DG_OLUTPG_MAX		DG_OLUTPG_MASK >> DG_OLUTPG_SHIFT

/*    u_short      res_1:5	reserved - reset to 0 */

/*    u_short      hoff		Horizontal Offset R/W */
/*				0-511 non-interleaved; 0-1022 interleaved */

#define DG_HOFF_OFFSET		0x08
#define DG_HOFF_MASK		0x01FF
#define DG_HOFF_MIN		DG_NO_MIN
#define DG_HOFF_MAX		DG_HOFF_MASK
#define DG_HOFF_INIT		0

/*    u_short      hact		Horizontal Active R/W */
/*                              4-4096 non-interleaved; 8-8192 interleaved */

#define DG_HACT_OFFSET		0x09
#define DG_HACT_MASK		0x0FFC
#define DG_HACT_MIN		4
#define DG_HACT_MAX		DG_HACT_MASK
#define DG_HACT_INIT		1024

/*    u_short      voff		Vertical Offset R/W */

#define DG_VOFF_OFFSET		0x0A
#define DG_VOFF_MASK		0x01FF
#define DG_VOFF_MIN		DG_NO_MIN
#define DG_VOFF_MAX		DG_VOFF_MASK
#define DG_VOFF_INIT		0

/*    u_short      vact		Vertical Active R/W */

#define DG_VACT_OFFSET		0x0B
#define DG_VACT_MASK		0x0FFF
#define DG_VACT_MIN		DG_NO_MIN
#define DG_VACT_MAX		DG_VACT_MASK
#define DG_VACT_INIT		768

/*    u_short      trgcon	Trigger Control & Status */
/* ---------------------------------------------------- */

#define DG_TRGCON_OFFSET	0x0C
#define DG_TRGCON_INIT		0x0000
				/* Trigger Disable */ 
				/* External Trigger */ 

/*    u_short      trigen:1	Trigger Enable R/W */

#define DG_TRIGEN_MASK		0x0001
#define DG_TRIGEN_SHIFT		0

/*    u_short      trigsrc:2	Trigger Source R/W */

#define DG_TRIGSRC_MASK		0x0002
#define DG_TRIGSRC_SHIFT	1
#define DG_TRIG_EXT		0	/* External Trigger */
#define DG_TRIG_SOFT		1	/* Software Trigger */

/*    u_short      trigcy:1	Trigger Cycle Status R-O */

#define DG_TRIGCY_MASK		0x0004
#define DG_TRIGCY_SHIFT		2
#define DG_IN_TRIGCY		0	/* Trigger Cycle in Progress */
#define DG_OUT_TRIGCY		1	/* No Trigger Cycle in Progress */

/*    u_short      res_1:2	reserved - reset to 0 */

/*    u_short      vbstat:1	Vertical Blank Status R-O */

#define DG_VBSTAT_MASK		0x0020
#define DG_VBSTAT_SHIFT		5
#define DG_IN_VB		0	/* IN camera vertical blank */
#define DG_OUT_VB		1	/* OUT of camera vertical blank */
/*    u_short      fld:1		Field Status R-O */


#define DG_FLD_MASK		0x0040
#define DG_FLD_SHIFT		6
#define DG_FLD_EVEN		0	/* camera in even field */
#define DG_FLD_ODD		1	/* camera in odd field */

/*    u_short      res_1:9	reserved - reset to 0 */

/*    u_short      lstat		Line Status R-O */

#define DG_LSTAT_OFFSET		0x0D
#define DG_LSTAT_MASK		0x0FFF
#define DG_LSTAT_MIN		DG_NO_MIN
#define DG_LSTAT_MAX		DG_VOFF_MASK

/*    u_short      swtrig	Software Trigger W-O */

#define DG_SWTRIG_OFFSET	0x0E
#define DG_SWTRIG_MASK		0x00FF
#define DG_SWTRIG_MIN		DG_NO_MIN
#define DG_SWTRIG_MAX		DG_SWTRIG_MASK		
#define DG_SWTRIG		0x00FF	/* Trigger per Software */

/*    u_short	  lut16[512]	16 by 16 Input LUT */ 	

#define	DG_LUT16_OFFSET		0x100
#define DG_LUT16_SIZE		0x100
#define DG_LUT16_MASK		DG_NO_MASK
#define DG_LUT16_MIN		DG_NO_MIN
#define DG_LUT16_MAX		DG_NO_MAX

/*    u_short	  lut8[512]	Optional 8 by 8 LUT */ 	

#define	DG_LUT8_OFFSET		0x200
#define DG_LUT8_SIZE		0x100
#define DG_LUT8_MASK		0x00FF
#define DG_LUT8_MIN		DG_NO_MIN
#define DG_LUT8_MAX		DG_LUT8_MASK

#endif /* linux && __KERNEL__ || __CONFIG__ */

#endif /* _AMDIGREG_H */

